@@ -123,6 +123,7 @@ def _wrap_func(func, msg, logger, version, remove_in):
     wrapper.__doc__ += _deprecation_docstring(func, msg, version, remove_in)
     return wrapper
 
+
 def _find_calling_frame(module_offset):
     g = [globals()]
     calling_frame = inspect.currentframe().f_back
@@ -135,6 +136,19 @@ def _find_calling_frame(module_offset):
             break
     return calling_frame
 
+
+def in_testing_environment():
+    """Return True if we are currently running in a "testing" environment
+
+    This currently includes if nose, nose2, pytest, or Sphinx are
+    running (imported).
+
+    """
+
+    return any(mod in sys.modules for mod in (
+        'nose', 'nose2', 'pytest', 'sphinx'))
+
+
 def deprecation_warning(msg, logger=None, version=None,
                         remove_in=None, calling_frame=None):
     """Standardized formatter for deprecation warnings
@@ -189,9 +203,18 @@ def deprecation_warning(msg, logger=None, version=None,
     if calling_frame is not None:
         info = inspect.getframeinfo(calling_frame)
         msg += "\n(called from %s:%s)" % (info.filename.strip(), info.lineno)
+        if deprecation_warning.emitted_warnings is not None:
+            if msg in deprecation_warning.emitted_warnings:
+                return
+            deprecation_warning.emitted_warnings.add(msg)
 
     logging.getLogger(logger).warning(msg)
 
+if in_testing_environment():
+    deprecation_warning.emitted_warnings = None
+else:
+    deprecation_warning.emitted_warnings = set()
+
 
 def deprecated(msg=None, logger=None, version=None, remove_in=None):
     """Decorator to indicate that a function, method or class is deprecated.
