@@ -10,7 +10,6 @@
 
 import logging
 import re
-import six
 import sys
 
 from pyomo.common.tempfiles import TempfileManager
@@ -168,7 +167,7 @@ class CPLEXDirect(DirectSolver):
         # least as far back as CPLEX 12.5.1 [the oldest version
         # supported by IBM as of 1 Oct 2020]
         if self.version() >= (12, 5, 1) \
-           and isinstance(self._log_file, six.string_types):
+           and isinstance(self._log_file, str):
             _log_file = (open(self._log_file, 'a'),)
             _close_log_file = True
         else:
@@ -221,8 +220,15 @@ class CPLEXDirect(DirectSolver):
                     self._solver_model.set_problem_type(self._solver_model.problem_type.QP)
                 else:
                     self._solver_model.set_problem_type(self._solver_model.problem_type.LP)
+
+            # if the user specifies a 'mipgap'
+            # set cplex's mip.tolerances.mipgap
+            if self.options.mipgap is not None:
+                self._solver_model.parameters.mip.tolerances.mipgap.set(float(self.options.mipgap))
             
             for key, option in self.options.items():
+                if key == 'mipgap': # handled above
+                    continue
                 opt_cmd = self._solver_model.parameters
                 key_pieces = key.split('_')
                 for key_piece in key_pieces:
